{library_calls/log_hlp.i}

run log-it (input "input_output_parameter_char_actually_works start").

{library_calls/libname.i}

def var i as int.
def var cmp-txt as char.
def var err-flag as logical.

/* these must be big enough to hold the changes otherwise you might see an error like */
/* Your LONGCHAR buffer has been overrun.  Memory has been corrupted. (14528) */
def var txt as character init "012345678901234567890123456789012345678901234567890123456789".
def var ltxt as longchar init "012345678901234567890123456789012345678901234567890123456789".

def var txt-array as character extent 3.
def var ltxt-array as longchar extent 3.

txt-array[1]  = "abcdefghijk".
txt-array[2]  = "abcdefghijk".
txt-array[3]  = "abcdefghijk".
ltxt-array[1] = "abcdefghijk".
ltxt-array[2] = "abcdefghijk".
ltxt-array[3] = "abcdefghijk".

run input_output_char_ptr_cdecl (input-output txt).
if txt ne "This is sample text!" then message "Expected no change to variable but txt = '" + txt + "'".
   
run input_output_char_ptr_cdecl (input-output ltxt).
if ltxt ne "This is sample text!" then message "Expected no change to variable but ltxt = '" + ltxt + "'".

run input_output_char_ptr_cdecl_array (input-output txt-array, input extent(txt-array)).

do i = 1 to extent(txt-array):
   cmp-txt = string((i - 1)).
   if txt-array[i] ne cmp-txt then message "Expected " + cmp-txt + " but instead found txt-array[" + string(i) + "] = '" + txt-array[i] + "'". 
end.

run input_output_char_ptr_cdecl_array (input-output ltxt-array, input extent(ltxt-array)).

do i = 1 to extent(ltxt-array):
   cmp-txt = string((i - 1)).
   if ltxt-array[i] ne cmp-txt then message "Expected " + cmp-txt + " but instead found ltxt-array[" + string(i) + "] = '" + ltxt-array[i] + "'". 
end.

/* empty string in array elements will work (the result is also an empty string, but the function is actually invoked with real pointers, but no contents are copied back!) */
do i = 1 to extent(txt-array):
   txt-array[i] = "".
end.

run input_output_char_ptr_cdecl_array (input-output txt-array, input extent(txt-array)).

do i = 1 to extent(txt-array):
   if txt-array[i] ne "" then message "Expected empty string but instead found txt-array[" + string(i) + "] = '" + txt-array[i] + "'". 
end.

/* empty string in longchar array elements causes an error */
/* A CHARACTER or LONGCHAR variable passed as an OUTPUT parameter to a DLL cannot be the Unknown value or be uninitialised. (12450) */
do i = 1 to extent(ltxt-array):
   ltxt-array[i] = "".
end.

run input_output_char_ptr_cdecl_array (input-output ltxt-array, input extent(ltxt-array)) no-error.
if not error-status:error or not error-status:get-number(1) eq 12450 then message "Expected an error passing an unknown longchar element in an output array parameter (got element 1 as '" + string(ltxt-array[1]) + "').".
   
/* unknown value in array elements raises an error condition */
/* A CHARACTER or LONGCHAR variable passed as an OUTPUT parameter to a DLL cannot be the Unknown value or be uninitialised. (12450) */
do i = 1 to extent(txt-array):
   txt-array[i] = ?.
end.

/* this silently generates an error when no-error is specified! */
/* A variable or array element passed as an INPUT or INPUT-OUTPUT parameter to a DLL cannot contain the Unknown value. (12272) */
err-flag = true.
do on error undo, leave:
   run input_output_char_ptr_cdecl_array (input-output txt-array, input extent(txt-array)) no-error.
   err-flag = false.
end.
if not err-flag or not error-status:error or not error-status:get-number(1) eq 12272 then message "Expected an error passing an unknown character element in an output array parameter (got error msg '" + error-status:get-message(1) + "').".

/* unknown value in longchar array elements raises an error condition */
/* A variable or array element passed as an INPUT or INPUT-OUTPUT parameter to a DLL cannot contain the Unknown value. (12272) */
do i = 1 to extent(ltxt-array):
   ltxt-array[i] = ?.
end.

err-flag = true.
do on error undo, leave:
   run input_output_char_ptr_cdecl_array (input-output ltxt-array, input extent(ltxt-array)) no-error.
   err-flag = false.
end.
if not err-flag or not error-status:error or not error-status:get-number(1) eq 12272 then message "Expected an error passing an unknown longchar element in an output array parameter (got element 1 as '" + error-status:get-message(1) + "').".
   
release external "{&libname}". 

procedure input_output_char_ptr_cdecl external "{&libname}" persistent:
   /* using this form makes no difference: def input-output parameter ptr as handle to character. */
   def input-output parameter ptr as character.
end.

procedure input_output_char_ptr_cdecl_array external "{&libname}" persistent:
   def input-output parameter ptr as character.
   def input parameter len as long.
end.

run log-it (input "input_output_parameter_char_actually_works end").
