{library_calls/log_hlp.i}

def var i1 as int.
def var i2 as int.
def var d1 as dec.
def var ptr as memptr.
def var txt as char.

run log-it (input "order_of_input_checks start").

/* this is the order of error checks on input, only the 1st failure found is reported (the checking process ends at that point): */
/* all of these checks occur before the attempt to load the library and before the attempt to lookup the function address */ 

/* 1. unknown value for any of the parms */
/* A variable or array element passed as an INPUT or INPUT-OUTPUT parameter to a DLL cannot contain the Unknown value. (12272) */
i1 = ?.
do on error undo, leave:
   /* this raises an error even in silent mode! */
   run proc1 (i1, i2) no-error.
end.
if not error-status:error or not error-status:get-number(1) eq 12272 then message "Expected error 12272 for passing unknown value (1). Got '" + error-status:get-message(1) + "'.".

i1 = 14.
i2 = ?.
do on error undo, leave:
   /* this raises an error even in silent mode! */
   run proc1 (i1, i2) no-error.
end.
if not error-status:error or not error-status:get-number(1) eq 12272 then message "Expected error 12272 for passing unknown value (2). Got '" + error-status:get-message(1) + "'.".

i1 = 14.
i2 = 30.
   
/* 2. the mode and type passed (in that order) for each of the parms is checked left to right (if not specified, input is the mode) */

/* Mismatched parameter types passed to procedure input_output_int32_3_parm_cdecl /home/gregs/p01007_Untitled1.ped. (3230) */
run proc3 (input i1, input-output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3230 then message "Expected error 3230 for passing an incompatible type (1). Got '" + error-status:get-message(1) + "'.".

run proc3 (input-output d1, ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3230 then message "Expected error 3230 for passing an incompatible type (2). Got '" + error-status:get-message(1) + "'.".
   
/* mixing literals in when you need all lvalues is the equivalent of input mode */
run proc3 (14.0, input-output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3230 then message "Expected error 3230 for passing an incompatible type (3). Got '" + error-status:get-message(1) + "'.".

/* you can't mark the literal as input-output because this is a compile error */
/* run proc3 (input-output 14.0, input-output ptr) no-error. */
   
/* Mismatch in the parameter datatypes in DLL procedure input_output_int32_3_parm_cdecl /home/gregs/p01007_Untitled1.ped. (3231) */
run proc3 (input-output i1, input-output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3231 then message "Expected error 3231 for passing an incompatible type. Got '" + error-status:get-message(1) + "'.".
   
/* strangely enough, this causes a memory violation: */
/* run proc1 (input-output i1, input-output ptr) no-error. */
   
/* 4. type-specific value checks for each of the parms left to right (first problem encountered causes the error) */
/* uninitialized memptr passed for any of the parms */
/* DLL procedure input_output_int32_3_parm_cdecl /home/gregs/p01007_Untitled1.ped using an uninitialised MEMPTR. (3233) */
/* integer size */
/* Value -129 does not fit in byte DLL datatype. (13712) */
i2 = -129.
run proc1 (input-output ptr, input-output i2) no-error.
if not error-status:error or not error-status:get-number(1) eq 3233 then message "Expected error 3233 for passing an uninitialized memptr as first parm. Got '" + error-status:get-message(1) + "'.".

run proc2 (input-output i2, input-output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 13712 then message "Expected error 13712 for passing an out of bounds int value as first parm. Got '" + error-status:get-message(1) + "'.".
   
def var big-i as int64 init 4294967295.
def var bigger-i as int64 init 9223372036854775807.

run proc5 (0, big-i, 2, 3, bigger-i, 5, 6, 7) no-error.
if not error-status:error or
   not error-status:num-messages eq 1 or
   not error-status:get-number(1) eq 13712 or
   not error-status:get-message(1) eq "Value 4294967295 does not fit in long DLL datatype. (13712)"
   then message "The order of input operations is wrong (1).".

run proc5 (0, bigger-i, 2, 3, big-i, 5, 6, 7) no-error.
if not error-status:error or
   not error-status:num-messages eq 1 or
   not error-status:get-number(1) eq 13712 or
   not error-status:get-message(1) eq "Value 9223372036854775807 does not fit in long DLL datatype. (13712)"
   then message "The order of input operations is wrong (2).".

/* 5. the wrong number of parms (too few or too many) */
/* Mismatched number of parameters passed to routine input_output_int32_3_parm_cdecl /home/gregs/p01007_Untitled1.ped. (3234) */
i2 = 0.
set-size(ptr) = 16.
run proc2 (input-output i2, input-output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3234 then message "Expected error 3234 for passing the wrong number of parms. Got '" + error-status:get-message(1) + "'.".
set-size(ptr) = 0.

run proc4 (input-output d1, input-output i1) no-error.
if not error-status:error or not error-status:get-number(1) eq 3234 then message "Expected error 3234 for passing the wrong number of parms. Got '" + error-status:get-message(1) + "'.".


run log-it (input "order_of_input_checks end").

procedure proc1 external "invalid libname, just some crap here":
   def input-output parameter i1 as memptr.
   def input-output parameter i2 as byte.
   def input-output parameter i3 as long.
end.

procedure proc2 external "invalid libname, just some crap here":
   def input-output parameter i1 as byte.
   def input-output parameter i2 as memptr.
   def input-output parameter i3 as long.
end.

procedure proc3 external "invalid libname, just some crap here":
   def input-output parameter i1 as double.
   def input-output parameter i2 as memptr.
   def input-output parameter i3 as long.
end.

procedure proc4 external "invalid libname, just some crap here":
   def input-output parameter i1 as double.
end.

procedure proc5 external "invalid libname, just some crap here":
   def input parameter i1 as long.
   def input parameter i2 as long.
   def input parameter i3 as long.
   def input parameter i4 as long.
   def input parameter i5 as long.
   def input parameter i6 as long.
   def input parameter i7 as long.
   def input parameter i8 as long.
end.

