
#include "helper.h"
#include <string.h>
#include <stdio.h>

extern int i_len;
extern int d_len;
extern int o_len;
extern char* text;
extern char* other;
extern int64_t num64;

#define INFUNC(typ, spec, cconv, fname)  DLL_EXPORT void cconv fname(typ val)\
{\
   flog(#fname "(" #typ " value = %" #spec ")\n", val);\
}

#define INIARRAY(typ, cconv, fname)  DLL_EXPORT void cconv fname(typ * val)\
{\
   flog(#fname "() with input values:");\
\
   for (int i = 0; i < i_len; i++)\
   {\
      flog(" 0x%016X,", val[i]);\
   }\
\
   flog("\n");\
}

#define INDARRAY(typ, cconv, fname, gname)  DLL_EXPORT void cconv fname(typ * val) { gname(val, d_len); } 

#define INDLIMITARRAY(typ, cconv, fname)  DLL_EXPORT void cconv fname(typ * val, int len)\
{\
   flog(#fname "() and input values:");\
\
   for (int i = 0; i < len; i++)\
   {\
      flog(" %g,", val[i]);\
   }\
\
   flog("\n");\
}

// CDECL functions
DLL_EXPORT void CDECL input_char_ptr_cdecl(char* ptr) { flog("input_char_ptr_cdecl(%s)\n", ptr ? ptr : "null"); }
DLL_EXPORT void CDECL input_char_ptr_cdecl_array(char* ptr[], int len)
{
   int n = 0;
   
   flog("input_char_ptr_cdecl_array() and input values:");
   
   for (int i = 0; i < len; i++)
   {
      if (ptr[i] != NULL)
      {
         flog(" '%s',", ptr[i]);
         n++;
      }
      else
         flog(" null,");
   }
   
   flog("; %i non-null elements processed\n", n);
}

DLL_EXPORT void CDECL input_void_ptr_cdecl(void* ptr)
{
   log_bytes("input_void_ptr_cdecl() and input bytes: ", "\n", ptr, o_len);
   memcpy(ptr, other, o_len);
}

DLL_EXPORT void CDECL input_void_ptr_cdecl_array(void* ptr[], int len)
{
   int n = 0;
   
   flog("input_void_ptr_cdecl_array() and input values: ");
   
   for (int i = 0; i < len; i++)
   {
      if (ptr[i] != NULL)
      {
         log_bytes("", "", ptr[i], o_len);
         n++;
      }
      else
         flog(" null,");
   }
   
   flog("- %i non-null elements processed\n", n);
}

DLL_EXPORT void CDECL input_char_ptr_ptr_cdecl(char** ptrptr) { flog("input_char_ptr_ptr_cdecl(%s)\n", *ptrptr); *ptrptr = text; }
DLL_EXPORT void CDECL input_void_ptr_ptr_cdecl(void** ptrptr) { log_bytes("input_void_ptr_ptr_cdecl() and input values: ", "\n", *ptrptr, o_len); *ptrptr = (void*) other; }
DLL_EXPORT void CDECL input_int64_ptr_cdecl(int64_t* ptr) {  flog("input_int64_ptr_cdecl(%" PRId64 ")\n", *ptr); *ptr = num64; }

DLL_EXPORT void CDECL input_int32_8_parm_cdecl(int32_t i1, int32_t i2, int32_t i3, int32_t i4, int32_t i5, int32_t i6, int32_t i7, int32_t i8)
{
   flog("input_int32_8_parm_cdecl() and input values: %i, %i, %i, %i, %i, %i, %i, %i\n", i1, i2, i3, i4, i5, i6, i7, i8);
}

DLL_EXPORT void CDECL input_int64_8_parm_cdecl(int64_t i1, int64_t i2, int64_t i3, int64_t i4, int64_t i5, int64_t i6, int64_t i7, int64_t i8)
{
   flog("input_int64_8_parm_cdecl() and input values: %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 "\n", i1, i2, i3, i4, i5, i6, i7, i8);
}

DLL_EXPORT void CDECL input_double_8_parm_cdecl(double d1, double d2, double d3, double d4, double d5, double d6, double d7, double d8)
{
   flog("input_double_8_parm_cdecl() and input values: %g, %g, %g, %g, %g, %g, %g, %g\n", d1, d2, d3, d4, d5, d6, d7, d8);
}

DLL_EXPORT void CDECL input_float_8_parm_cdecl(float f1, float f2, float f3, float f4, float f5, float f6, float f7, float f8)
{
   flog("input_float_8_parm_cdecl() and input values: %g, %g, %g, %g, %g, %g, %g, %g\n", f1, f2, f3, f4, f5, f6, f7, f8);
}

DLL_EXPORT void CDECL input_char_ptr_8_parm_cdecl(char* c1, char* c2, char* c3, char* c4, char* c5, char* c6, char* c7, char* c8)
{
   flog("input_char_ptr_8_parm_cdecl() and input values: '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s'\n", c1, c2, c3, c4, c5, c6, c7, c8);
}

INFUNC(uint8_t, u, CDECL, input_uint8_cdecl)
INIARRAY(uint8_t, CDECL, input_uint8_cdecl_array)

INFUNC(int16_t, i, CDECL, input_int16_cdecl)
INIARRAY(int16_t, CDECL, input_int16_cdecl_array)

INFUNC(uint16_t, u, CDECL, input_uint16_cdecl)
INIARRAY(uint16_t, CDECL, input_uint16_cdecl_array)

// PRIi32 can't be used here because it is a macro and won't expand right
INFUNC(int32_t, i, CDECL, input_int32_cdecl)
INIARRAY(int32_t, CDECL, input_int32_cdecl_array)

// PRIu32 can't be used here because it is a macro and won't expand right
INFUNC(uint32_t, u, CDECL, input_uint32_cdecl)
INIARRAY(uint32_t, CDECL, input_uint32_cdecl_array)

// PRId64 can't be used here because it is a macro and won't expand right, hopefully lld works on windows
INFUNC(int64_t, lld, CDECL, input_int64_cdecl)
INIARRAY(int64_t, CDECL, input_int64_cdecl_array)

INFUNC(float, g, CDECL, input_float_cdecl)
INDLIMITARRAY(float, CDECL, input_float_cdecl_limit_array)
INDARRAY(float, CDECL, input_float_cdecl_array, input_float_cdecl_limit_array)

INFUNC(double, g, CDECL, input_double_cdecl)
INDLIMITARRAY(double, CDECL, input_double_cdecl_limit_array)
INDARRAY(double, CDECL, input_double_cdecl_array, input_double_cdecl_limit_array)

// STDCALL functions (only emit on Windows 32-bit)
#ifdef WINDOWS32
INFUNC(uint8_t, u, STDCALL, input_uint8_stdcall)
INIARRAY(uint8_t, STDCALL, input_uint8_stdcall_array)

INFUNC(int16_t, i, STDCALL, input_int16_stdcall)
INIARRAY(int16_t, STDCALL, input_int16_stdcall_array)

INFUNC(uint16_t, u, STDCALL, input_uint16_stdcall)
INIARRAY(uint16_t, STDCALL, input_uint16_stdcall_array)

// PRIi32 can't be used here because it is a macro and won't expand right
INFUNC(int32_t, i, STDCALL, input_int32_stdcall)
INIARRAY(int32_t, STDCALL, input_int32_stdcall_array)

// PRIu32 can't be used here because it is a macro and won't expand right
INFUNC(uint32_t, u, STDCALL, input_uint32_stdcall)
INIARRAY(uint32_t, STDCALL, input_uint32_stdcall_array)

// PRId64 can't be used here because it is a macro and won't expand right, hopefully lld works on windows
INFUNC(int64_t, lld, STDCALL, input_int64_stdcall)
INIARRAY(int64_t, STDCALL, input_int64_stdcall_array)

INFUNC(float, g, STDCALL, input_float_stdcall)
INDLIMITARRAY(float, STDCALL, input_float_stdcall_limit_array)
INDARRAY(float, STDCALL, input_float_stdcall_array, input_float_stdcall_limit_array)

INFUNC(double, g, STDCALL, input_double_stdcall, copy_double_value)
INDLIMITARRAY(double, STDCALL, input_double_stdcall_limit_array)
INDARRAY(double, STDCALL, input_double_stdcall_array, input_double_stdcall_limit_array)

DLL_EXPORT void STDCALL input_char_ptr_stdcall(char* ptr) { flog("input_char_ptr_stdcall(%s)\n", ptr); }
DLL_EXPORT void STDCALL input_char_ptr_stdcall_array(char* ptr[], int len)
{
   int n = 0;
   
   flog("input_char_ptr_stdcall_array() and input values:");
   
   for (int i = 0; i < len; i++)
   {
      if (ptr[i] != NULL)
      {
         flog(" '%s',", ptr[i]);
         n++;
      }
      else
         flog(" null,");
   }
   
   flog("; %i non-null elements processed\n", n);
}

DLL_EXPORT void STDCALL input_void_ptr_stdcall(void* ptr) { log_bytes("input_void_ptr_stdcall() and input values: ", "\n", ptr, o_len); memcpy(ptr, other, o_len); }
DLL_EXPORT void STDCALL input_char_ptr_ptr_stdcall(char** ptrptr) { flog("input_char_ptr_ptr_stdcall(%s)\n", *ptrptr); *ptrptr = text; }
DLL_EXPORT void STDCALL input_void_ptr_ptr_stdcall(void** ptrptr) { log_bytes("input_void_ptr_ptr_stdcall() and input values: ", "\n", *ptrptr, o_len); *ptrptr = (void*) other; }

DLL_EXPORT void STDCALL input_int32_8_parm_stdcall(int32_t i1, int32_t i2, int32_t i3, int32_t i4, int32_t i5, int32_t i6, int32_t i7, int32_t i8)
{
   flog("input_int32_8_parm_stdcall() and input values: %i, %i, %i, %i, %i, %i, %i, %i\n", i1, i2, i3, i4, i5, i6, i7, i8);
}

DLL_EXPORT void STDCALL input_int64_8_parm_stdcall(int64_t i1, int64_t i2, int64_t i3, int64_t i4, int64_t i5, int64_t i6, int64_t i7, int64_t i8)
{
   flog("input_int64_8_parm_stdcall() and input values: %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 ", %" PRId64 "\n", i1, i2, i3, i4, i5, i6, i7, i8);
}

DLL_EXPORT void STDCALL input_double_8_parm_stdcall(double d1, double d2, double d3, double d4, double d5, double d6, double d7, double d8)
{
   flog("input_double_8_parm_stdcall() and input values: %g, %g, %g, %g, %g, %g, %g, %g\n", d1, d2, d3, d4, d5, d6, d7, d8);
}

DLL_EXPORT void STDCALL input_float_8_parm_stdcall(float f1, float f2, float f3, float f4, float f5, float f6, float f7, float f8)
{
   flog("input_float_8_parm_stdcall() and input values: %g, %g, %g, %g, %g, %g, %g, %g\n", f1, f2, f3, f4, f5, f6, f7, f8);
}

DLL_EXPORT void STDCALL input_char_ptr_8_parm_stdcall(char* c1, char* c2, char* c3, char* c4, char* c5, char* c6, char* c7, char* c8)
{
   flog("input_char_ptr_8_parm_stdcall() and input values: '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s'\n", c1, c2, c3, c4, c5, c6, c7, c8);
}
#endif
